//! ATmega128 Uart Library
/*!
* Author comment : Uart Library
*
* Author : Alex 23rd.
*
* Filename : uart.c
*
* Contents : Atmega128 Uart 0,1 Library
*
* Date : 2005-02-27
*
* Version : v0.2
*
* This program belong to Inha University Robot Research.
* Non permitted commercial use is highly prohibited.
*
* Modification :
* 2005-02-02 first implementation
* 2005-02-27 Seperate UART0,1 related functions
*
* TODO
*  - UART0,1 RX/TX Interrupt Library, Register Setting.
*/

#include <avr/io.h>
#include "uart.h"

PUBLIC void
uart1_init(u32 baud)
{
	baud =	(F_CPU / (16L*baud)) - 1;
	
	//uart1 register setting
	UBRR1H = (u08)(baud>>8);
	UBRR1L = (u08)(baud);
	UCSR1B = (1<<7) | (1<<4) | (1<<3) | (0<<2); 
	UCSR1C = (1<<3) | (1<<2) | (1<<1) | (0<<5) | (0<<4);	
}

PUBLIC void
uart0_init(u32 baud)
{
	baud =	(F_CPU / (16L*baud)) - 1;
	
	//uart1 register setting
	UBRR0H = (u08)(baud>>8);
	UBRR0L = (u08)(baud);
	UCSR0B = (1<<7) | (1<<4) | (1<<3) | (0<<2); 
	UCSR0C = (1<<3) | (1<<2) | (1<<1) | (0<<5) | (0<<4);	
}

PRIVATE u08
uart1_recv(void)
{
    u08 dummy;
	while(!(UCSR1A & (1<<7)))
	{
        dummy = UDR1;
		return dummy;
	}
	
	return '\0';
}

PRIVATE u08
uart0_recv(void)
{
    u08 dummy;
	while(!(UCSR0A & (1<<7)))
	{
        dummy = UDR0;
		return dummy;
	}
	
	return '\0';
}

PRIVATE void
uart0_send(u08 data)
{
	while(!(UCSR0A & (1<<5)));
	UDR0 = data;
}

PRIVATE void
uart1_send(u08 data)
{
	while(!(UCSR1A & (1<<5)));
	UDR1 = data;
}

PUBLIC u08
getch_(u08 device)
{
	u08	 dummy;
	
	#ifdef _DEBUG_UART_
	printf_("here is getch_\n");
	#endif
	//while(!(UCSR1A & (1<<7)));
	//dummy = UDR1;
	if(device == 0)
	{
        #ifdef _DEBUG_UART_
        printf_("in system\n");
        #endif
        while( !(UCSR0A&(1<<7)) )
        {
            #ifdef  _DEBUG_UART_
            printf_("%x\n",UCSR0A);
            #endif
        }
        #ifdef _DEBUG_UART_
        printf_("after while\n");
        #endif
    	dummy = UDR0;
    	#ifdef _DEBUG_UART_
    	printf_("mm\n");
    	#endif
        //dummy = uart0_recv();
    }
    else
    {
        dummy = uart1_recv();
    }
	return dummy;
}

PUBLIC u08
getche_(u08 device)
{
	u08 dummy;
	if(device == 0)
	{
        dummy = uart0_recv();
        putchar_(0,dummy);
    }
    else
    {
        dummy = uart1_recv();
        putchar_(1,dummy);
    }
	//while(!(UCSR1A & (1<<7)));
	//dummy = UDR1;
	//putchar_(dummy);
	return dummy;
}

PUBLIC void
putchar_(u08 device,u08 data)
{
	if(data != '\n')
	{
        if(device == 0)
        {
            uart0_send(data);
        }
        else
        {
            uart1_send(data);
        }        
    }
	else if(data == '\n')
	{
        if(device == 0)
        {
            uart0_send(0x0d);
    		uart0_send(0x0a);
        }
        else
        {
            uart1_send(0x0d);
    		uart1_send(0x0a);
        }
		// if new line
	}
}

PUBLIC void
puts_(u08 device,u08* str)
{
	while(*str)
	{
		if(*str == '\n')
		{
            if(device == 0)
            {
                putchar_(0,0x0d);
    			putchar_(0,0x0a);
            }
            else
            {
                putchar_(1,0x0d);
    			putchar_(1,0x0a);
            }
			str++;
		}
		else
		{
            if(device == 0)
            {
                putchar_(0,*str);
            }
            else
            {
                putchar_(1,*str);
            }
		str++;
		}
	}
}

/*
PUBLIC void
puthex_(u08 device,u32 value)
{
	u08 i;
	u08 c;
	u08 buffer[9];
	u08* buf;
	
	buf=buffer;
	
	*buf++='\0';
	
	c=0;
	
	do
	{
		i=value%16;
		if(i<10)
		{
			*buf++ = i + '0';
		}else if(i>=10)
		{
			*buf++ = (i-10) + 'A';
		}
		c++;
	}while((value/=16)>0);
	
	if(c==1)
	{
	  *buf++ = '0';
	}
	
	*buf--;
	
	while(*buf != '\0')
	{
        if(device == 0)
        {
            putchar_(0,*buf--);
        }
        else
        {
            putchar_(1,*buf--);
        }
	}
}

PUBLIC void
putdec_(u08 device,u32 value)
{
	PRIVATE u08 i;
	PRIVATE u08 c;
	PRIVATE u08 buffer[9];
	PRIVATE u08* buf;
	
	buf=buffer;
	
	*buf++='\0';
	
	c=0;
	
	do
	{
		i=value%10;
		*buf++ = i + '0';
		c++;
	}while((value/=10)>0);
	
	*buf--;
	
	while(*buf != '\0')
	{
		if(device == 0)
        {
            putchar_(0,*buf--);
        }
        else
        {
            putchar_(1,*buf--);
        }
	}
}
*/
/*
PUBLIC void
printhex_(u08 ch)
{
  putchar_(hexcode[(ch >> 4)]);
  putchar_(hexcode[(ch & 0x0f)]); 
}*/
